@@ -1,6 +1,7 @@
 from __future__ import unicode_literals
 
 import logging
+
 from funcy.py3 import cat
 
 from . import locked
@@ -76,7 +77,7 @@ def _cloud_status(
         force=True,
         remote=remote,
         jobs=jobs,
-    )["local"]
+    )
 
     ret = {}
     status_info = self.cloud.status(used, jobs, remote=remote)
